/**
 * @overview WMF Package Class
 * @author Jan "Waffle" Christophersen
 * @copyright (c) Jan "Waffle" Christophersen
 */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _wmfUtility = require('./wmfUtility');

var _wmfUtility2 = _interopRequireDefault(_wmfUtility);

global.log = _wmfUtility2['default'].log;

/**
 * The base Package class provides basic functionality for multiple packages to work and interact.
 */

var WMFPackage = (function () {
  /**
   * Creates a new Package
   */

  function WMFPackage(name) {
    var _this = this;

    _classCallCheck(this, WMFPackage);

    /** @type {string} */
    this.name = name;
    /** @type {boolean} */
    this.initialized = false;

    // probe the gtamp namespace for existing packages with that name
    if (gtamp && gtamp.packages) {
      if (gtamp.packages.hasOwnProperty(name)) {
        throw new Error('a package named ' + name + ' already exists.');
      }
    }

    // announce ourself.
    if (typeof gtamp.packages === 'undefined') {
      gtamp.packages = new Map();
    }
    gtamp.packages.set(name, this);
    events.Call('WMFPackageLoaded', this);

    // get ready to update our namespace whenever a new package is being loaded.
    events.Add('WMFPackageLoaded', function () {
      if (_this.initialized) {
        _this._updateNamespace();
      } else {
        _this._tryStart();
      }
    });
    events.Add('WMFPackageInitialized', function () {
      if (!_this.initialized) {
        _this._tryStart();
      }
    });

    process.nextTick(this._tryStart.bind(this));
  }

  /**
   * Updates the namespaces.
   */

  _createClass(WMFPackage, [{
    key: '_updateNamespace',
    value: function _updateNamespace() {
      var name = global._wmfConfig.namespaceName;

      // create the global namespace
      if (typeof global[name] === 'undefined') {
        global[name] = {};
      }

      gtamp.packages.forEach(function (pkg, packageName) {
        global[name][packageName] = pkg;
      });
    }

    /**
     * Tries to start the package.
     */
  }, {
    key: '_tryStart',
    value: function _tryStart() {
      if (this.initialized) {
        return;
      }
      log.debug('WMFPackage#tryStart \'' + this.name + '\'');

      if (this.dependencies.length === 0) {
        this.init();
        return;
      }

      if (typeof gtamp.packages === 'undefined') {
        return;
      }

      // lets see if all dependencies are loaded.
      for (var idx = 0; idx < this.dependencies.length; idx++) {
        if (!gtamp.packages.has(this.dependencies[idx])) {
          return;
        }
        if (!gtamp.packages.get(this.dependencies[idx]).initialized) {
          return;
        }
      }
      this._updateNamespace();
      this.init();
    }

    /**
     * Returns the package's dependencies.
     *
     * @type {Array.<string>}
     */
  }, {
    key: 'init',

    /**
     * Initializes the package.
     */
    value: function init() {
      this.initialized = true;
      log.info('package ' + this.name + ' has been initialized.');

      events.Call('WMFPackageInitialized', this);
    }

    /**
     * Requests a File (module, either *.js or *.json) from the package's directory. The name of the WMFPackage needs to be set to the name of the folder the package resides in.
     *
     * @param {string} file - path to the file
     * @returns {Promise.<Object>}
     */
  }, {
    key: 'requestModule',
    value: function requestModule(file) {
      var _this2 = this;

      return new Promise(function (resolve, reject) {
        if (!_wmfUtility2['default'].path.extname(file).startsWith('.js')) {
          return reject(new Error('only *.js and *.json files are allowed.'));
        }

        var dir = file;
        if (_wmfUtility2['default'].path.isAbsolute(dir)) {
          dir = _wmfUtility2['default'].path.relative(process.cwd() + '/packages/' + _this2.name + '/', dir);
        }
        resolve(require(process.cwd() + '/packages/' + _this2.name + '/' + dir));
      });
    }

    /**
     * Requests all modules of a directory (*.js or *.json files). The name of the WMFPackage needs to be set to the name of the folder the package resides in.
     *
     * @param {string} directory - path to the directory
     * @param {boolean} [recursive=true] - include subdirectories
     * @returns {Promise.<Map.<Object>>}
     */
  }, {
    key: 'requestDirectory',
    value: function requestDirectory(directory) {
      var _this3 = this;

      var recursive = arguments.length <= 1 || arguments[1] === undefined ? true : arguments[1];

      return new Promise(function (resolve, reject) {
        var dir = directory;
        if (_wmfUtility2['default'].path.isAbsolute(dir)) {
          dir = _wmfUtility2['default'].path.relative(process.cwd() + '/packages/' + _this3.name + '/', dir);
        }
        dir = process.cwd() + '/packages/' + _this3.name + '/' + dir;

        _wmfUtility2['default'].readdir(dir, recursive).then(function (files) {
          var promises = [];
          files.forEach(function (file) {
            if (!_wmfUtility2['default'].path.extname(file).startsWith('.js')) {
              return undefined;
            }

            promises.push(_this3.requestModule(_wmfUtility2['default'].path.join(directory, file)).then(function (mod) {
              return [file, mod];
            }));
          });

          Promise.all(promises).then(function (modules) {
            resolve(new Map(modules));
          })['catch'](reject);
        })['catch'](reject);
      });
    }
  }, {
    key: 'dependencies',
    get: function get() {
      return [];
    }
  }]);

  return WMFPackage;
})();

exports['default'] = WMFPackage;
module.exports = exports['default'];
//# sourceMappingURL=wmfPackage.js.map